----------------------------------------------------------------------------------
-- Autor: Jess Eduardo Mndez Rosales
--
---- Description: Aplicacin de ejemplo para utilizar el DAC como generador de funciones.
--                Se pueden generar hasta 4 funciones las cuales estn almacenadas en una
--                memoria ROM, cada funcin cuenta con 160 muestras. La frecuencia es fija.
---------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_unsigned.ALL;

entity TOP is

PORT(
		CLK   : IN  STD_LOGIC; -- Reloj FPGA a 100MHz
		RESET : IN  STD_LOGIC; -- Reset de DAC
		SDI   : OUT STD_LOGIC; -- Pin para protocolo serial SPI
		CS    : OUT STD_LOGIC; -- Chip select del DAC
		SCK   : OUT STD_LOGIC; -- Reloj SPI
		LDAC  : OUT STD_LOGIC; -- Pin LDAC de MCP4822
		CH    : IN  STD_LOGIC; -- Switch selector de canal
		FUN   : IN  STD_LOGIC_VECTOR(2 DOWNTO 0) -- Switches para seleccin de funcin
);

end TOP;

architecture Behavioral of TOP is

COMPONENT LIB_DAC_AVANXE7_REVA is

Port ( 
    CLK      : IN STD_LOGIC; 
    RESET    : IN STD_LOGIC; 
    CH       : IN STD_LOGIC; 
    DATA_DAC : IN STD_LOGIC_VECTOR(11 DOWNTO 0);
    SDI      : OUT STD_LOGIC;
    CS       : OUT STD_LOGIC;
    SCK      : OUT STD_LOGIC;
    LDAC     : OUT STD_LOGIC;
    CONV_FIN : OUT STD_LOGIC 
);

end COMPONENT LIB_DAC_AVANXE7_REVA;

COMPONENT MEM_MUESTRAS is

PORT(
      a : IN integer;
     spo : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
);

end COMPONENT MEM_MUESTRAS;

-- Seales para memoria ROM
signal spo : std_logic_vector(15 downto 0) := (others => '0'); -- Valor de las muestras, slo se toman los bits del 11 al 0
signal add : std_logic_vector(11 downto 0) := (others =>'0'); -- Indica en qu posicin de la memoria se mepezar a leer
signal a   : integer := 0; -- Direccin de la memoria ROM a leer


-- Seales para lib_dac_avanxe7_reva
signal conv_fin : std_logic := '0';

-- Seales para mquina de estados que controla librera del DAC y ROM
signal edo : integer range 0 to 100 := 0;


begin

u1: lib_dac_avanxe7_reva
port map( 
    clk, reset, ch, spo(11 downto 0), sdi, cs, sck, ldac, conv_fin
);

u2:  mem_muestras
port map(
      a, spo
);

-- Proceso que le indica la posicin de la ROM a leer
process(clk)
begin
	if rising_edge(clk) then
        if conv_fin = '1' then -- Cuando finaliza la conversin se pasa a la siguiente posicin de la ROM
			a <= a+1;
			if a >= 159+conv_integer(add) then-- Cuando el contador "a" llega al valor mximo entonces se le asigna el valor de "add"
				a <= conv_integer(add);
			end if;
        end if;
	end if;
end process;

-- Proceso que selecciona la funcin dependiendo el valor de los switches
process(fun)
begin
    -- Dependiendo el valor de "fun" es el valor que se le asigna a "add", cada posicin de "add" 
    -- contiene la primer muestra de cada funcin 
	if fun = "000" then
		add <= x"000";

	elsif fun = "001" then
		  add <= "000010100000";

	elsif fun = "010" then
		  add <= "000101000000";

	 elsif fun = x"011" then
		  add <= "000111100000";

	 else
		  add <= "001010000000";

	end if;
end process;


end behavioral;
